const fs = require("fs");
const path = require("path");
const yaml = require("js-yaml");
const findRoot = require("find-root");
const { Command } = require("@oclif/command");
const { Config, State } = require("@dotdev/next-config");

class GenerateGitLabCICommand extends Command {
  async run() {
    const state = State.resolve();
    const config = Config.resolve();
    const environment = config.getEnvironment();

    const targetThemes = state.get("targetThemes", []);

    if (0 === targetThemes.length) {
      this.error("No target themes, run next ci first");
    }

    const projectRoot = findRoot(process.cwd());

    const parentYamlPath = path.resolve(projectRoot, ".gitlab-ci.yml");
    const childYamlPath = path.resolve(projectRoot, ".gitlab-ci-child.yml");

    const parentYaml = yaml.load(fs.readFileSync(parentYamlPath));

    const childYaml = {
      ...(parentYaml.image ? { image: parentYaml.image } : {}),
      ...(parentYaml.cache ? { cache: parentYaml.cache } : {}),
      ...(parentYaml.before_script
        ? { before_script: parentYaml.before_script }
        : {}),
    };

    for (const targetTheme of targetThemes) {
      childYaml[
        `deploy-${"main" === targetTheme.role ? "live" : targetTheme.id}`
      ] = {
        stage: "deploy",
        variables: {
          SHOPIFY_ENV: environment.handle,
          SHOPIFY_THEME_ID: targetTheme.id,
          SHOPIFY_KEY: environment.credentials.apiKey,
          SHOPIFY_PASSWORD: environment.credentials.password,
        },
        script: [
          "npx next ci --env $SHOPIFY_ENV --api-key $SHOPIFY_KEY --password $SHOPIFY_PASSWORD --theme $SHOPIFY_THEME_ID",
          "npm run deploy",
        ],
      };
    }

    fs.writeFileSync(childYamlPath, yaml.dump(childYaml, { lineWidth: -1 }));

    this.log(`child pipelines written to ${childYamlPath}`);
  }
}

GenerateGitLabCICommand.description = "generate gitlab ci file";

module.exports = GenerateGitLabCICommand;
