const { Command, flags } = require("@oclif/command");
const { Deployer } = require("@dotdev/next-deployer");
const { Config, State } = require("@dotdev/next-config");

class MigrateCommand extends Command {
  async run() {
    const flags = this.parse(MigrateCommand).flags;

    const config = Config.resolve();
    const state = State.resolve();

    const deployer = new Deployer(config);

    const themes = await deployer.getThemes();

    const sourceTheme = themes.find((theme) => "main" === theme.role);

    let themesToMigrate = [];

    if (flags.created) {
      themesToMigrate = state.get("createdThemes", []);
    }

    for (const theme of themesToMigrate) {
      this.log(`Copying theme settings from master for theme ${theme.name}...`);
      await deployer.copyThemeSettings(sourceTheme.id, theme.id);
    }
  }
}

MigrateCommand.description = "migrate settings from one theme to another";

MigrateCommand.flags = {
  created: flags.boolean({ description: "migrate themes recently created" }),
};

module.exports = MigrateCommand;
